/**
 * ConditionProcessor - Conditional selection of choices
 *
 * @author	Jonathan Roewen
 */
#ifndef CONDITION_PROCESSOR_H
#define CONDITION_PROCESSOR_H

#include "AimlProcessor.h"
#include "Memory.h"
#include "Kernel.h"

#include <string>

using namespace std;

class ConditionProcessor : public AimlProcessor
{
public:
	~ConditionProcessor() { }
	
	string getName() const {
		return "condition";
	}
	string getVersion() const {
		return "1.0";
	}
	string process(Match *m, PElement e, Responder *r, const string &id) {
		string name = e->getAttribute("name");
		string value = e->getAttribute("value");
		velement li;
		e->getChildren("li", &li);
		for (unsigned int ix = 0; ix < li.size(); ++ix) {
			string n = li[ix]->getAttribute("name");
			if (n.empty()) {
				n = name;
			}
			string v = li[ix]->getAttribute("value");
			string c = li[ix]->getAttribute("contains");
			string bw = li[ix]->getAttribute("beginswith");
			string ew = li[ix]->getAttribute("endswith");
			string mem = Memory::getValue(n, id);
			if (!v.empty()) {
				if (v == mem) {
					return Kernel::process(m, li[ix], r, id);
				}
			}
			if (!c.empty()) {
				if (mem.find(c) != string::npos) {
					return Kernel::process(m, li[ix], r, id);
				}
			}
			if (!bw.empty()) {
				if (mem.find(bw) == 0) {
					return Kernel::process(m, li[ix], r, id);
				}
			}
			if (!ew.empty()) {
				if (mem.length() >= ew.length() && mem.rfind(ew) == mem.length() - ew.length()) {
					return Kernel::process(m, li[ix], r, id);
				}
			}
			if (c.empty() && v.empty() && bw.empty() && ew.empty()) {
				return Kernel::process(m, li[ix], r, id);
			}
		}
		if (!name.empty() && !value.empty()) {
			string mem = Memory::getValue(name, id);
			if (value == mem) {
				return Kernel::process(m, e, r, id);
			}
		}
		return "";
	}
};

#endif