/**
 * IrcResponder - Interacts with users via IRC
 *
 * @author	Jonathan Roewen, Taras Glek
 */
#include "IrcResponder.h"

#include "StringTokenizer.h"
#include "Kernel.h"
#include "Utils.h"

#include <iostream>

using namespace std;

IrcResponder::IrcResponder(const string &server, const int &port, const string &nick, const string &channel) {
	this->server = server; this->port = port;
	sock = new Socket(server, port, this);
	connect(nick, channel);
}

IrcResponder::~IrcResponder() {
	if (sock != NULL) {
		disconnect();
		delete sock;
	}
	sock = NULL;
}

string IrcResponder::respond(Match *m, PElement element, const string &id) {
	if (element->getNamespace() != "irc") {
		return "";
	} else {
		string tag = element->getTagname();
		if (tag == "channel") {
			return channel;
		} else
		if (tag == "lastnick") {
			return lastnick;
		} else
		if (tag == "command") {
			q.push(Kernel::process(m, element, this, id));
		} else {
			cout << tag << " : " << Kernel::process(m, element, this, id) << endl;
		}
	}
	return "";
}

void IrcResponder::execute() {
	while (!q.empty()) {
		sendCommand(q.front());
		q.pop();
	}
}

void IrcResponder::process() {
	string line;
	string::size_type linebreak = buffer.find("\n");
	while (linebreak != string::npos) {
		line = buffer.substr(0, linebreak);
		buffer = trim(buffer.erase(0, linebreak), "\n");
		linebreak = buffer.find("\n");
		//	Got a single line to process
		string sender, message = line, command, params;
		if (line[0] == ':') {
			string prefix = line.substr(1);
			sender = trim(prefix.substr(0, prefix.find("!")));
			message = prefix.substr(prefix.find(" "));
			command = trim(message.substr(1, message.find(" ", 1)));
		} else {
			command = trim(message.substr(0, message.find(" ")));
		}
		if (command.empty()) {
			cout << "empty command" << endl;
			break;
		}
		params = trim(message.substr(message.find(" ", 1)));
		
		if (command == "PRIVMSG") {
			string target = params.substr(0, params.find(" "));
			string msg = params.substr(params.find(':') + 1);
			string reply_msg, reply_to;
			lastnick = sender;
			if (target[0] == '#') {
				//	Message was to a channel the bot is in
				reply_to = target;
				if (msg.find(nick) == 0) {
				//	msg = msg = " ";
					reply_msg = msg.substr(msg.find(nick) + nick.length()+  ((msg.length() > nick.length())? 1:0));
				} else
				if (msg.find(nick) != string::npos) {
					//	Message contained the bot's nick .. transform nick to bot name
					string::size_type minSearch = 0;
					while ((minSearch = msg.find(nick, minSearch)) != string::npos) {
						string m_head = msg.substr(0, minSearch);
						string m_tail = msg.substr(minSearch + nick.length());
						msg = m_head + Kernel::respond("BOT NAME", sender, this) + m_tail;
					}
					reply_msg = msg;
				}
			} else {
				//	Message was to the bot
				reply_to = sender;
				reply_msg = msg;
			}
			if (!reply_msg.empty()) {
				StringTokenizer lines(Kernel::respond(reply_msg, sender, this), "\n");
				while (lines.hasMoreTokens()) {
					send("PRIVMSG " + reply_to + " :" + lines.nextToken());
				}
			}
			execute();	//	I guess
		//	respond(reply_to, sender);
		} else
		if (command == "PING") {
			send("PONG " + params);
		} else
		if (command == "INVITE") {
			string channel = trim(params.substr(params.find(':') + 1));
			send("JOIN " + channel);
		} else {
		//	cout << "COMMAND: " << command << "." << endl;
		//	cout << "PARAMS: " << params << "." << endl;
		}
	}
}

void IrcResponder::sendCommand(const string &command) {
	string::size_type pos = command.find(" ");
	if (pos != string::npos) {
		send(toUpper(command.substr(0, pos)) + command.substr(pos));
	} else {
		send(command);
	}
}

void IrcResponder::received(string &recv) {
	buffer += recv;
	process();
}

void IrcResponder::send(const string &send) {
	if (sock == NULL) {
		return;
	}
	sock->write(send + "\r\n");
}

void IrcResponder::connect(const string &nick, const string &channel) {
	this->nick = nick; this->channel = channel;
	send("USER joobjoob j-alice.sourceforge.net " + server + " :joob joob");
	send("PASS joobjoobnz@yahoo.co.nz");
	send("NICK " + nick);
	send("JOIN " + channel);
	sock->socketLoop();
}

void IrcResponder::disconnect() {
	send("QUIT :joob joob");
}
