/**
 * Socket
 *
 * Judge: why you extern "C" the whole lot? why at all? I didn't
 * Winsock2 seems no different .. which means windows sockets
 * just plain suck :-(
 * @author	Jonathan Roewen, Taras Glek
 */
#include "Socket.h"

#ifdef WIN32
	#define __WIN32__ WIN32
	#include <winsock.h>
#else
	#include <netdb.h>
	#include <sys/socket.h>
#endif

#include <list>
#include <iostream>

using namespace std;

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

static list<Socket *> sockets;

#ifdef __WIN32__
	extern "C" {
		int winsock_handle = -1;
		int socket_count = 0;
		
		void win32_init_sockets(void) {
			WSADATA d;
			
			if (winsock_handle == -1) {
				winsock_handle = WSAStartup(MAKEWORD(2, 0), &d);
			}
		}
	}
#endif

void Socket::socketLoop() {
	static int running=0;
	/*1 socketloop per application*/
	if(running)
		return;
	running = 1;
	int result;
	Socket *s;
	int sd;
	list<Socket*>::iterator it;
	while (!sockets.empty()) {
		fd_set input_sockets;
		
		FD_ZERO(&input_sockets);
		for (it = sockets.begin(); it != sockets.end(); it++) {
			s = *it;
			if (s->isAlive()) {
				FD_SET(s->getSD(), &input_sockets);
			}
		}
		struct timeval timeout;
		
		timeout.tv_sec = 2;
		timeout.tv_usec = 500000;
		
		result = select(FD_SETSIZE, &input_sockets, (fd_set *)0, (fd_set *)0, &timeout);
		switch (result) {
			case 0:
				continue;	//	Timeout
			case -1:
				perror("select");
				running = 0;
				return;	//	This'll do for moment
		}
		for (it = sockets.begin(); it != sockets.end(); it++) {
			s = *it;
			sd = s->getSD();
			if (sd != -1 && FD_ISSET(sd, &input_sockets)) {
				s->incoming();
			}
		}
	}
	running = 0;
}
void Socket::addListeningSocket(Socket *s){
	sockets.push_back(s);
}
void Socket::init(){
	#ifdef __WIN32__
		win32_init_sockets();
	#endif
}

Socket::Socket(const string &host, const int port, SocketListener *listn, bool autoconnect) {
	buffer_size = 5000;
	buffer = new char[buffer_size];
	this->host = host;
	this->port = port;	
	sd = -1;
	if(autoconnect)
		connectPeer();
	setSocketListener(listn);
	this->addListeningSocket(this);
}

Socket::~Socket() {
	sockets.remove(this);
	terminate();
	delete [] buffer;
}
void Socket::setAsync(){
/*	int opts;

	opts = fcntl(sd,F_GETFL);
	if (opts < 0) {
		perror("fcntl(F_GETFL)");
		return;
	}
	opts = (opts | O_NONBLOCK);
	if (fcntl(sd,F_SETFL,opts) < 0) {
		perror("fcntl(F_SETFL)");
		return;
	}
*/	return;
}

void Socket::terminate(){
	if(isAlive()){
		close(sd);
		sd = -1;
	} 
}

void Socket::connectPeer(){
	int rc;
	struct sockaddr_in self, server;
	struct hostent *h;
	init();
	//	Desirable for this to be non-blocking
	h = gethostbyname(host.c_str());
	if (h == NULL) {
		perror(host.c_str());
		sd = -1;
		return;
	}
	
	server.sin_family = h->h_addrtype;
	memcpy((char *)&server.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
	server.sin_port = htons(port);
	
	sd = socket(AF_INET, SOCK_STREAM, 0);
	if (sd < 0) {
		perror("Can't open socket");
		sd = -1;
		return;
	}
	
	self.sin_family = AF_INET;
	self.sin_addr.s_addr = htonl(INADDR_ANY);
	self.sin_port = htons(0);
	
	rc = bind(sd, (struct sockaddr *)&self, sizeof(self));
	if (rc < 0) {
		perror("Can't connect");
		sd = -1;
		return;
	}
	
	rc = connect(sd, (struct sockaddr *)&server, sizeof(server));
	if (rc < 0) {
		perror("Can't connect");
		sd = -1;
		return;
	}

}
int Socket::getSD() {
	return sd;
}

void Socket::setSocketListener(SocketListener *sl) {
	listener = sl;
}

int Socket::read_raw(void *buffer, int buf_size) {
	int n;
	memset(buffer, 0x0, buf_size);
	//deal with dead/inactive socket
	if(sd == -1)
		return 0;
	#ifdef __WIN32__
		n = recv(sd, (char FAR *)buffer, buf_size, 0);
	#else
		n = recv(sd, (char *)buffer, buf_size, 0);
	#endif
	if (n < 0) {
		perror("Can't receive data");
		sd = -1;
	} else if (n == 0) {
		printf("Connection closed by client\n");
		sd = -1;
		//	Judge: why is this printf .. not perror?
	}
	
	return n;
}

int Socket::read(string &str) {
	int n;
	int buf_size = buffer_size;
	if (buf_size > 1) {
		buf_size--;
	}
	n = read_raw(buffer, buf_size);
	buffer[n] = 0;
	str = string(buffer);
	if (listener != NULL) {
		listener->received(str);
		cout << "listener == " << listener << endl;
	}else{
		cout << "listener == NULL";
	}
	return n;
}

void Socket::incoming(){
	string s;
	read(s);
}

int Socket::write_raw(const void *data, int len) {
	if(sd == -1)
		return 0;
	#ifdef __WIN32__
		return send(sd, (char FAR *)data, len, 0);
	#else
		return send(sd, (const char *)data, len, 0);
	#endif
}

int Socket::write(const string &str) {
	return write_raw(str.c_str(), str.length());
}

TokenProxyListener::TokenProxyListener(/*SocketListener &listener,*/const string &token = "\r\n"):
	/*forward(listener),*/separator(token){
	cout << "New Proxy listener" << endl;
}

void TokenProxyListener::received(string &str){
	cout << "Received" << endl;
	cout << str << endl;
}
	