/**
 * Socket - So we can use the Internet!
 *
 * @author	Jonathan Roewen, Taras Glek
 */
#ifndef SOCKET_H
#define SOCKET_H

#include <string>

using namespace std;

class SocketListener;

class Socket
{
public:
	Socket(const string &, const int, SocketListener *s = NULL,bool autoconnect=true);
	Socket(int socket, SocketListener *s = NULL):sd(socket),listener(s){}
	virtual ~Socket();
	
	virtual int read(string &);
	void connectPeer();
	int write(const string &);
	void setSocketListener(SocketListener *);
	int getSD();
	bool isAlive(){return sd != -1;}
	/*this method manages all of the active sockets in the application*/
	static void socketLoop();
	/*do non-blocking io..this method is a big api FIXME*/
	void setAsync();
	/*closes the connection*/
	void terminate();
	/* create a socket from an existing connection*/
	SocketListener* getSocketListener(){ return listener;}
protected:
	void init();
	void setPort(int p){port = p;}
	/*put the socket into the socketloop*/
	void addListeningSocket(Socket *);
	/*gets triggered by socketLoop when there is data to be read*/
	virtual void incoming();

protected:
	int read_raw(void *, int);
	int write_raw(const void *, int);
	
	int buffer_size, sd;
	string host;
	int port;
	SocketListener *listener;
	char *buffer;
};

class SocketListener
{
public:
	virtual void received(string &) = 0;
};

/** cool class, this one only notifies the listener when a whole token 
has arrived...aka a newline..its meant to be used as a filler 
aka setSocketListener(new TokenProxyListener(*this)) or something
*/
class TokenProxyListener:public SocketListener{
public:
	TokenProxyListener(/*SocketListener &listener,*/const string &token = "\r\n");
	virtual ~TokenProxyListener() { }
	void received(string &);	
private:
	string buffer,separator;
	int lastPos;
	//SocketListener &forward;
};
#endif  